import {describe, it} from 'mocha';
import {Encryptor} from '.';
import {assert, expect} from 'chai';
import fc from 'fast-check';
import {randomBytes} from 'crypto';
import {KeyLengthError} from './types';
import {keyBits} from './config';

describe('Encryptor', () => {
  it('fails given a key of an incorrect length', () => {
    const badKey = Buffer.from('badkey');
    const badLen = badKey.length * 8;

    expect(() => new Encryptor(badKey)).to.throw(
      KeyLengthError,
      badLen + '-bit'
    );
  });

  it('Should encrypt messages given any text', async () => {
    const encryptor = new Encryptor(randomBytes(keyBits / 8));
    fc.assert(
      fc.property(fc.string(), plainText => {
        const ciphertext = encryptor.encrypt(plainText);
        testBase64EncodedCiphertext(ciphertext);
      })
    );
  });
});

const testBase64EncodedCiphertext = (ciphertext: string, blockSize = 16) => {
  // Prefix is added
  assert(
    ciphertext.startsWith('Encrypted:'),
    'Text starts with encryption prefix'
  );
  // Larger than the size of the IV and ciphertext
  expect(ciphertext.length).to.be.greaterThan(blockSize * 2);
  // Text is base64 encoded
  assert(/[a-zA-Z0-9+/]+=*$/.test(ciphertext), 'Ciphertext is base64 encoded');
};
