/**
 * Liza cryptographic library
 *
 * This library provides opinionated cryptographic operations for Liza.  It
 * is intended to straddle both the source and destination---the library is
 * used both by Liza itself, and is intended for use by downstream systems
 * that consume data from Liza, allowing for Liza's implementation to vary
 * over time without affecting downstream systems.
 *
 * To get started as a user of this library outside of Liza, see
 * [`README.md`](../index.html).
 *
 * @module
 */

export {Decryptor} from './decrypt';
export {Encryptor} from './encrypt';

export {
  decryptorFromEncryptedSessionKeys,
  encryptorFromEncryptedSessionKeys,
  encryptSessionKey,
  decryptSessionKey,
  createSessionKey,
  EncodedSessionKey,
  EncryptedSessionKey,
} from './session';

// `assertKeyLength` is performed automatically by the `Encryptor` and
// `Decryptor` constructors, but callers may prefer to check their keys
// prior to construction.
export {assertKeyLength, Key} from './types';

export {ProtectedString} from './types/protected-string';

export {
  KeyLengthError,
  SessionKeyDecryptionError,
  InvalidPrefixError,
  DecryptionError,
  MessageLengthError,
} from './types/error';

export {isPossiblyEncrypted} from './util';
