export class KeyLengthError extends Error {
  constructor(expected_bits: number, given_bits: number) {
    const str =
      'Invalid key length ' +
      `(expected ${expected_bits}-bit, given ${given_bits}-bit)`;
    super(str);
    this.name = 'MessageLengthError';
    fixErrorPrototype(this, KeyLengthError);
  }
}

export class SessionKeyDecryptionError extends Error {
  constructor() {
    const str = 'No valid session keys available';
    super(str);
    this.name = 'SessionKeyDecryptionError';
    fixErrorPrototype(this, SessionKeyDecryptionError);
  }
}

export class InvalidPrefixError extends Error {
  constructor() {
    const str = 'Incorrect prefix on string';
    super(str);
    this.name = 'InvalidPrefixError';
    fixErrorPrototype(this, InvalidPrefixError);
  }
}

export class DecryptionError extends Error {
  constructor() {
    const str = 'Error on ciphertext decryption';
    super(str);
    this.name = 'DecryptionError';
    fixErrorPrototype(this, DecryptionError);
  }
}

export class MessageLengthError extends Error {
  constructor() {
    const str = 'Invalid message length';
    super(str);
    this.name = 'MessageLengthError';
    fixErrorPrototype(this, MessageLengthError);
  }
}

// Extending error overrides the objects prototype
// This sets it back to the class instance
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const fixErrorPrototype = (_this: unknown, _class: any) => {
  Object.setPrototypeOf(_this, _class.prototype);
};
