import {keyBits} from '../config';
import {KeyLengthError} from './error';

/** Binary key for cryptographic primitives. */
export type Key = Buffer;

/**
 * Throw a `KeyLengthError` if the provided `key` does not have a length of
 * 256 bits.
 */
export const assertKeyLength = (key: Key) => {
  const givenBits = key.length * 8;

  if (givenBits !== keyBits) {
    throw new KeyLengthError(keyBits, givenBits);
  }
};
