import {blockSizeBytes, ciphertextPrefix} from '../config';

/** Basic regex to estimate whether an input might be base64-encoded */
const base64Regex = /^[-A-Za-z0-9+/]*={0,2}$/;

/**
 * Makes an estimation as to whether a provided string is encrypted or not
 *
 * Checks prefix, if remaining data is in base64, and the size of the block
 *
 * @param x unknown item to check
 * @returns boolean indicating whether or not the text provided is encrypted
 */
export function isPossiblyEncrypted(x: string) {
  return (
    x.startsWith(ciphertextPrefix) &&
    base64Regex.test(x.substring(ciphertextPrefix.length)) &&
    x.length > blockSizeBytes + ciphertextPrefix.length
  );
}
