/**
 * Provide means to clear bucket values
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import type {BucketData, EmptyBucketData} from './Bucket';

/**
 * Generate update to clear bucket fields
 *
 * @param prefix - bucket field prefix, e.g. "meta."
 * @param fields - field names and their bucket values
 *
 * @return cleared values
 */
export const bucketClearer =
  (prefix = '') =>
  (fields: BucketData): EmptyBucketData =>
    Object.keys(fields).reduce((result: EmptyBucketData, field: string) => {
      const currField = fields[field];
      if (!currField) return result;
      if (currField.length > 0) {
        result[prefix + field] = new Array(currField.length).fill('');
      }

      return result;
    }, {});
