/**
 * Liza Base Bucket Layer
 *
 * This class is simply a wrapper for a QuoteDataBucket
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework
 *
 *  Liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
import {ClassData} from '../../client/Cmatch';
import {BucketData, BucketFieldValue} from '../Bucket';
import {QuoteDataBucket} from '../QuoteDataBucket';
import {LayeredBucket} from './LayeredBucket';

export class BaseLayer implements LayeredBucket {
  constructor(private readonly _bucket: QuoteDataBucket) {}

  /**
   * Update class data
   *
   * This class has no need to update the class data so this is a no-op
   *
   * @param data - the data to update
   */
  public updateClassData(_data: ClassData): this {
    return this;
  }

  public getData(): BucketData {
    return this._bucket.getData();
  }

  public getDataByName(name: string): BucketFieldValue | null {
    return this._bucket.getDataByName(name);
  }
}
