/**
 * Liza Bucket Layer
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework
 *
 *  Liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
import {ClassData} from '../../client/Cmatch';
import {BucketData, BucketFieldValue} from '../Bucket';

export interface LayeredBucket {
  /**
   * Return raw bucket data
   *
   * TODO: remove; breaks encapsulation
   *
   * @return raw bucket data
   */
  getData: () => BucketData;

  /**
   * Returns the data for the requested field
   *
   * @param name - name of the field (with or without trailing brackets)
   *
   * @return data for the field
   */
  getDataByName: (name: string) => BucketFieldValue | null;

  /**
   * Update the class data
   *
   * @param classData - the class data to update
   */
  updateClassData: (classData: ClassData) => LayeredBucket;
}
