/**
 * Liza createLayeredBucket
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework
 *
 *  Liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import {BucketLayerData} from '../../program/Program';
import {QuoteDataBucket} from '../QuoteDataBucket';
import {LayeredBucket} from './LayeredBucket';
import {BaseLayer} from './BaseLayer';
import {DefaultingLayeredBucket} from './DefaultingLayeredBucket';

/**
 * Create a layered bucket
 *
 * @param layer     - a layer to build upon
 * @param layerData - layer definitions
 *
 * @returns a layered bucket
 */
export function createLayeredBucket(
  layer: LayeredBucket | QuoteDataBucket,
  layerData: BucketLayerData[]
): LayeredBucket {
  const nextData: BucketLayerData | undefined =
    layerData?.[layerData.length - 1];

  // If layer isnt an instance of BucketLayer, create a base
  if (!('updateClassData' in layer)) {
    layer = new BaseLayer(layer);
  }

  if (!nextData) {
    return layer;
  }

  const {defaults, when} = nextData;

  // Right now we only support defaulting layers, in the future there may be
  // more implementations
  const nextLayer = new DefaultingLayeredBucket(layer, when, defaults);

  return createLayeredBucket(nextLayer, layerData.slice(0, -1));
}
