/**
 * Bucket utility functions
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import {BucketData} from '../Bucket';

export const getPrefixes =
  (suffix: string) =>
  (data: BucketData): string[] =>
    Object.keys(data || {})
      .filter(key => key.endsWith(suffix))
      .map(key => key.slice(0, -suffix.length))
      .filter(key => key !== '');

export const getPrefixesTrue =
  (suffix: string) =>
  (data: BucketData) =>
  (prefixes: string[]): string[] => {
    return prefixes.filter(prefix => {
      const val = data?.[prefix + suffix]?.[0];

      return Array.isArray(val) ? !!val?.[0] : !!val;
    });
  };
