/**
 * Liza client
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework
 *
 *  Liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import {Ui} from '../ui/Ui';
import {ClientQuote} from './quote/ClientQuote';
import {Nav} from './nav/Nav';
import {ElementStyler} from '../ui/ElementStyler';
import {Program} from '../program/Program';
import {ClientInterface} from './ClientInterface';
import {CmatchData} from './Cmatch';
import {Step} from '../step/Step';
import {PositiveInteger} from '../numeric';
import {EventEmitter} from 'events';

/**
 * Controller for the program client
 */
export declare class Client extends EventEmitter implements ClientInterface {
  /**
   * Handles navigation
   */
  nav: Nav;

  /**
   * Styles DOM elements
   */
  elementStyler: ElementStyler;

  /**
   * Holds the Program object generated from the XML
   */
  program: Program;

  /**
   * Returns the UI object
   */
  getUi(): Ui;

  getDocument(): HTMLDocument;

  /**
   * Returns the current quote
   */
  getQuote(): ClientQuote;

  /**
   * Handle error events
   *
   * Ideally, this should never happen. This method indicates an error that
   * could not be properly handled by another part of the system. Let the user
   * know that this should not be happening and trigger our own error event.
   */
  handleError(e: Error): undefined;

  /**
   * Handles client-side events
   */
  handleEvent(
    event_name: string,
    data: Record<string, unknown>,
    callback?: (data: any) => void,
    error_callback?: (err: Error) => void
  ): Client;

  /**
   * Hooks quote for performing validations on data change
   *
   * @param diff - Diff to validate
   */
  validateChange(diff: any): void;

  /**
   * Whether there is a current save event
   */
  isSaving(): boolean;

  /**
   * Whether we are an internal user
   */
  isInternal(): boolean;

  /**
   * Whether there is a navigation event in progress
   */
  isNavigating(): boolean;

  /**
   * Update cmatch data where necessary
   *
   * @param cmatch - the updated class match data
   */
  cmatchUpdate(cmatch: CmatchData): void;

  /**
   * Whether there is a current step loaded
   */
  hasCurrentStep: () => boolean;

  /**
   * Get the current step id
   *
   * @returns the current step id
   */
  getCurrentStepId: () => PositiveInteger;
}
