/**
 * Contains concrete RestDataApi class
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import {DataApi, DataApiInput, DataApiResult} from './DataApi';
import {RestDataApiStrategy} from './RestDataApiStrategy';

/**
 * Retrieve data over a RESTful API
 */
export class RestDataApi implements DataApi {
  /**
   * Initialize data API
   *
   * @param url      - service URL
   * @param strategy - request strategy
   */
  constructor(
    private readonly _url: string,
    private readonly _strategy: RestDataApiStrategy
  ) {}

  /**
   * Request data from the service
   *
   * @param data     - request params
   * @param callback - server response callback
   */
  request(data: DataApiInput, callback: NodeCallback<DataApiResult>): this {
    this._strategy.requestData(this._url, data, callback);

    return this;
  }
}
