/**
 * Formats program bucket data
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import {QuoteDataBucket} from '../bucket/QuoteDataBucket';
import {FieldClassMatcher} from '../field/FieldClassMatcher';
import {Program} from '../program/Program';

/**
 * Formats program bucket data
 *
 * This takes a document and formats the bucket data in a
 * structured manner of steps, groups and fields metadata
 */
export declare class DocumentProgramFormatter {
  /**
   * Initialize document formatter
   *
   * @param {Program}           program       active program
   * @param {FieldClassMatcher} class_matcher class/field matcher
   */
  constructor(program: Program, class_matcher: FieldClassMatcher);

  /**
   * Formats document data into structure similar to program
   *
   * @param {Bucket} bucket document bucket
   *
   * @return {Promise.<Object>} a promise of a data object
   */
  format(bucket: QuoteDataBucket): Promise<unknown>;
}
