/**
 *  Dullahan Client
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
import {Program} from '../../program/Program';
import {PositiveInteger} from '../../numeric';
import {ClientInterface} from '../../client/ClientInterface';
import {ClientQuote} from '../../client/quote/ClientQuote';
import {CmatchData} from '../../client/Cmatch';

/**
 * Serves as a client for dullahan
 */
export class DullahanClient implements ClientInterface {
  private _step_id: PositiveInteger = <PositiveInteger>0;

  constructor(
    readonly program: Program,
    private readonly _quote: ClientQuote
  ) {}

  getQuote(): ClientQuote {
    return this._quote;
  }

  handleError(error: Error): void {
    throw error;
  }

  /**
   * This is unimplemented for dullahan
   */
  cmatchUpdate(_: CmatchData): void {}

  /**
   * Get the current step
   *
   * This returns a step object whereas the ui client returns a stepui object,
   * because of the current implementation, this is acceptable because classes
   *
   */
  hasCurrentStep(): boolean {
    return this._step_id === 0;
  }

  /**
   * Get the current step id
   *
   * @returns the current step id
   */
  getCurrentStepId(): PositiveInteger {
    return this._step_id;
  }

  /**
   * This is unimplemented for dullahan
   */
  getUi(): undefined {
    return undefined;
  }

  /**
   * Handles client-side events
   */
  handleEvent(
    event_name: string,
    data: Record<string, unknown>,
    _callback?: (data: any) => void,
    _error_callback?: (err: Error) => void
  ): DullahanClient {
    // FIXME: this should work correctly when DEV-9728 is implemented
    if (event_name !== 'show' && event_name !== 'hide')
      this.handleError(
        new Error(
          `Unhandled event: ${event_name} with data ${JSON.stringify(data)}`
        )
      );

    return this;
  }
}
