/**
 *  System Controller
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
import {Db} from 'mongodb';
import {EventEmitter} from 'events';
import {Request, Response} from 'express';

export const system = {
  /**
   * Make sure the system is online
   *
   * @param request  - request object
   * @param response - response object
   */
  healthcheck:
    (emitter: EventEmitter) =>
    (db: Db) =>
    async (_: Request, response: Response) => {
      try {
        const ping = await db.command({ping: 1});

        if (ping.ok) {
          response.status(204).send();
        } else {
          throw new Error('Failed to ping DB');
        }
      } catch (e) {
        emitter.emit('error', 'Failed health check');
        response.status(503).send('Service Unavailable');
      }
    },
};
