import * as TE from 'fp-ts/TaskEither';
import {DocumentData} from '../../server/db/MongoServerDao';
import {MongoServerDao} from '../../server/db/MongoServerDao';
import {Program} from '../../program/Program';
import {ServerSideQuote} from '../../server/quote/ServerSideQuote';
import {UserSession} from '../../server/request/UserSession';
import {initNewDocument, cleanDocument} from '../../server/quote/loader';
import {pipe} from 'fp-ts/function';
import {FeatureFlag} from '../../system/flags/FeatureFlag';
import {KekService, ServiceSecretMap} from '../../server/service/KekService';

type QuoteMaker = (program: Program) => (id: number) => ServerSideQuote;

export type AsyncQuoteFactory = (
  dao: MongoServerDao
) => (
  quote_maker: QuoteMaker
) => (
  program: Program
) => (
  session: UserSession
) => (
  feature_flag: FeatureFlag
) => (doc_data: DocumentData) => (keks: ServiceSecretMap) => any;

/**
 * Create and init a new quote/document
 *
 * @param dao         - mongo dao
 * @param quote_maker - logic for how to create quote
 * @param session     - user session
 * @param doc_data    - preliminary data to load onto the document
 * @param program     - relevant program
 *
 * @return a quote
 */
export const quote_factory: AsyncQuoteFactory =
  dao =>
  quote_maker =>
  program =>
  session =>
  feature_flag =>
  doc_data =>
  keks => {
    const liza_kek = KekService.getLatestInteralKek(keks);
    return pipe(
      initNewDocument(dao)(quote_maker(program))(program)(session)(keks)(
        feature_flag
      )(doc_data),
      TE.chain(cleanDocument({liza_kek})(program)),
      TE.getOrElse(e => {
        throw e;
      })
    )();
  };
