/**
 * Logs client-side errors
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import {isObject} from '../../type_safety';
import {PriorityLog} from '../log/PriorityLog';
import {UserRequest} from '../request/UserRequest';

export function route(request: UserRequest, log: PriorityLog) {
  if (!request.getSession().isLoggedIn()) {
    return Promise.resolve(false);
  }

  if (!request.getUri().match(/^\/?clienterr/)) {
    return Promise.resolve(false);
  }

  request.consumePostData(data => {
    const data_obj = isObject(data) ? data : {};

    log.log(
      PriorityLog.PRIORITY_ERROR,
      '[Client-side error] %s "%s" %s:%s  "%s" "%s" :: %s',
      request.getSession().agentId() || '',
      data_obj.file + '' || '',
      data_obj.line + '' || '-',
      data_obj.column + '' || '-',
      data_obj.message + '' || '<no message>',
      request.getUserAgent() || '',
      data_obj.stack ? JSON.stringify(data_obj.stack) : '<no stack trace>'
    );
  });

  // we handled the request
  request.end();
  return Promise.resolve(true);
}
