/**
 * Manages DataAPI requests and return data
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import {ServerSideQuote} from '../quote/ServerSideQuote';
import {UserRequest} from '../request/UserRequest';
import {Store} from '../../store/Store';
import {DataApiManager} from '../../dapi/DataApiManager';
import {TokenDao} from '../token/TokenDao';
import {generateTokenId} from '../token/Token';

/* eslint-disable @typescript-eslint/no-var-requires */
const createServerDataApiFactory = require('../request/ServerDataApiFactory');
const createDataApiManager = require('../../dapi/DataApiManager');
/* eslint-enable @typescript-eslint/no-var-requires */

/**
 * Create new server-based DataApiManager
 *
 * @param origin_url - HTTP_ORIGIN_URL
 * @param dapi_conf  - dapi configuration
 * @param collection - database collection
 * @param ts_ctor    - timestamp constructor
 * @param apis       - API definitions
 * @param request    - Node HTTP request
 * @param quote      - current quote for request
 *
 * @return data API manager
 */
export const createDapiManager =
  (origin_url: string) =>
  (dapi_conf: Store) =>
  (token_dao: TokenDao) =>
  (gcp_file_path: string) =>
  (
    apis: CommonObject,
    request: UserRequest,
    quote: ServerSideQuote
  ): DataApiManager =>
    createDataApiManager(
      createServerDataApiFactory(
        origin_url,
        request,
        dapi_conf,
        quote,
        token_dao,
        generateTokenId,
        gcp_file_path
      ),
      apis
    );
