/* TODO auto-generated eslint ignore, please fix! */
/* eslint no-var: "off" */
/**
 * Augments a quote with additional data for use by the quote server
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

var Class = require('easejs').Class,
  Quote = require('../../quote/Quote'),
  BaseQuote = require('../../quote/BaseQuote');

module.exports = Class('ServerSideQuote')
  .implement(Quote)
  .extend(BaseQuote, {
    /**
     * Program version
     * @type {string}
     */
    'private _pver': '',

    /**
     * Credit score reference number
     * @type {number}
     */
    'private _creditScoreRef': 0,

    /**
     * Agency number associated to quote
     * @type {string}
     */
    'private _agency_number': null,

    /**
     * Unix timestamp containing date of first premium calculation
     * @type {number}
     */
    'private _rated_date': 0,

    /**
     * Metabucket
     * @type {Bucket}
     */
    'private _metabucket': null,

    /**
     * Rating Data Bucket
     * @type {Bucket}
     */
    'private _rate_bucket': null,

    /**
     * The number of rate retries that have been attempted
     * @type {number}
     */
    'private _retry_attempts': 0,

    /**
     * Fields which will not persist when saving the quote
     * @type {Array.of<string>}
     */
    'private _nonpersistent_fields': [],

    /**
     * Session key
     *
     * This is an encrypted session key that will be used to encrypt data
     * before it is placed into the bucket
     */
    'private _session_key': null,

    /**
     * Downstream Session Keys
     *
     * A key value store of the various downstream systems set for
     * encryption and their respective session keys
     */
    'private _downstream_session_keys': null,

    /**
     * Latest KEK Versions
     *
     * A key value store of the various kek versions stored and used
     * on the quote
     */
    'private _kek_versions': null,

    /** State of each currently applicable field */
    'private _field_state': undefined,

    'public setProgramVersion': function (version) {
      this._pver = '' + version;
      return this;
    },

    'public getProgramVersion': function () {
      return this._pver;
    },

    'public setCreditScoreRef': function (ref) {
      this._creditScoreRef = +ref;
      return this;
    },

    'public getCreditScoreRef': function () {
      return this._creditScoreRef;
    },

    /**
     * Gets the retail agency number
     *
     * @return agency number
     */
    'public getAgencyNumber': function () {
      return this._agency_number;
    },

    /**
     * Sets the retail agency number
     *
     * @param agency_num represents the retail agency
     */
    'public setAgencyNumber': function (agency_num) {
      this._agency_number = '' + (agency_num || '');
      return this;
    },

    /**
     * Set the timestamp of the first time quote was rated
     *
     * @param {number} timestamp Unix timestamp representing first rated date
     *
     * @return {Quote} self
     */
    'public setRatedDate': function (timestamp) {
      // do not overwrite date if it exists
      if (this._rated_date === 0) {
        this._rated_date = +timestamp || 0;
      }

      return this;
    },

    /**
     * If the quote has been rated
     *
     * @return {boolean} has been rated
     */
    'public getRatedDate': function () {
      return this._rated_date;
    },

    /**
     * Metadata bucket
     *
     * @return {Bucket}
     */
    'public getMetabucket': function () {
      return this._metabucket;
    },

    /**
     * Set metadata bucket
     *
     * @return {ServerSideQuote} self
     */
    'public setMetabucket': function (metabucket) {
      this._metabucket = metabucket;
    },

    /**
     * Set metabucket data
     *
     * @param {Object} data key/value data
     *
     * @return {ServerSideQuote} self
     */
    'public setMetadata': function (data) {
      if (!this._metabucket) {
        throw Error('No metabucket available for #setMetadata');
      }

      this._metabucket.setValues(data);
      return this;
    },

    /**
     * Set rating bucket
     *
     * @param {Bucket} bucket the rate bucket to set
     */
    'public setRateBucket': function (bucket) {
      this._rate_bucket = bucket;

      return this;
    },

    /**
     * Get rating bucket
     *
     * @return {Bucket}
     */
    'public getRateBucket': function () {
      return this._rate_bucket;
    },

    /**
     * Set rating data
     *
     * @param {Object.<string,Array>} data rating data
     */
    'public setRatingData': function (data) {
      if (!this._rate_bucket) {
        throw Error('No rating bucket available for #setRatingData');
      }

      this._rate_bucket.setValues(data);

      return this;
    },

    /**
     * Get rating data
     *
     * @return {Object.<string,Array>} rating data
     */
    'public getRatingData': function () {
      if (!this._rate_bucket) {
        throw Error('No rating bucket available for #setRatingData');
      }

      return this._rate_bucket.getData();
    },

    /**
     * Set the number of retries attempted
     *
     * @return {ServerSideQuote} self
     */
    'public setRetryAttempts': function (attempts) {
      this._retry_attempts = attempts;

      return this;
    },

    /**
     * Get the number of retries attempted
     *
     * @return {number} the number of attempts that have been made
     */
    'public getRetryAttempts': function () {
      return this._retry_attempts;
    },

    /** Classify this quote and update field state */
    classify() {
      const program = this.getProgram();
      const class_data = program.classify(this.getBucket().getData());

      /**
       * Force class matrices into vectors
       *
       * If `xs` is not a matrix, the return value is unchanged from `xs`.
       *
       * The idea is that fields should never have matrix classification
       * data, because bucket data is always a vector.  This was overly
       * accommodating for convenience, and has bit us; in the future, the
       * compiler will force things to vectors for us, and we won't have to
       * worry about this.
       */
      const matrixToVec = xs =>
        !isArray(xs) ? xs : xs.map(x => (isArray(x) ? +x.some(v => !!v) : x));

      this.getLayeredBucket().updateClassData(class_data);

      const {isArray} = Array;
      const qvisState = visq => class_data =>
        Object.fromEntries(
          Object.entries(class_data)
            .map(
              ([c, {is, indexes}]) =>
                is && visq[c] && [visq[c], matrixToVec(indexes)]
            )
            .filter(isArray)
        );
      const filtered = qvisState(program.whenq)(class_data);

      this._field_state = filtered;

      return class_data;
    },

    /**
     * State of each currently applicable field
     *
     * This is updated by `#classify`, and so will be `undefined` if that
     * has not yet been invoked.  Since classifying is a potentially
     * expensive operation, do not expect that field state will always be
     * available, and do not expect that the classifier would be
     * automatically invoked to acquire it.
     */
    getFieldState() {
      return this._field_state;
    },

    /** Set applicability of quote as it existed the last save */
    setLastPersistedFieldState(state) {
      this._last_persisted_field_state = state;
      return this;
    },

    /** Applicability of each field the last time the quote was saved */
    getLastPersistedFieldState() {
      return this._last_persisted_field_state;
    },

    /**
     * Determines if user session is authorized to access quote
     */
    isAuthorized(session) {
      const agent_id = this.getAgentId();
      const meta = this.getMetabucket().getData();
      const agency_num = session.agencyNumber();
      const username = session.userName();
      const access_groups =
        meta !== undefined && meta['liza_access_groups'] !== undefined
          ? meta['liza_access_groups']
          : undefined;

      const access_email =
        meta !== undefined && meta['liza_access_email'] !== undefined
          ? meta['liza_access_email']
          : undefined;

      return (
        // internal users can access any quote
        session.isInternal() === true ||
        // TODO: this is transitional, use of agentId is deprecated
        (session.agentId() === agent_id && access_groups === undefined) ||
        // the user agency exists in the quote access group
        (agency_num !== undefined &&
          Array.isArray(access_groups) &&
          access_groups.includes(agency_num)) ||
        (username !== undefined &&
          Array.isArray(access_email) &&
          access_email.includes(username))
      );
    },

    /**
     * Set the nonpersistent fields
     *
     * @param {Array.of<string>} fields nonpersistent fields
     *
     * @returns self for method chaining
     */
    setNonpersistent(fields) {
      this._nonpersistent_fields = fields;

      return this;
    },

    /**
     * Return keys which should not persist to the database
     *
     * @return {Array.of<string>} nonpersistent keys
     */
    getNonpersistent() {
      return this._nonpersistent_fields;
    },

    /**
     * Recalculate the step delta given a new step id
     *
     * @param {number} step_id step id to recalculate with
     */
    refreshStepDelta(step_id) {
      return this.setData({
        __liza_step_delta: [this._getStepDelta(step_id)],
        __liza_step_id_current: ['' + step_id],
      });
    },

    /**
     * Determine the delta given a new step id
     *
     * @param {number} step_id step id to recalculate with
     *
     * @returns the delta
     */
    'private _getStepDelta': function (step_id) {
      const current_step = parseInt(
        this.getDataByName('__liza_step_id_current')[0]
      );

      if (step_id < current_step) {
        return '-1';
      } else if (step_id > current_step) {
        return '1';
      }

      return '0';
    },

    /**
     * Set lizas encrypted session key
     *
     * @param {string} key encrypted session key
     */
    'public setSessionKeys': function (keys) {
      this._session_key = keys;
      return this;
    },

    /**
     * Set the downstream systems encrypted session keys
     *
     * @param {string} key encrypted session key
     */
    'public setDownstreamSessionKeys': function (key_list) {
      const downstream_keys = {...key_list};
      // Remove liza from this context to only save
      // downstream keys
      delete downstream_keys.liza;
      this._downstream_session_keys = downstream_keys;
      return this;
    },

    /**
     * Sets the latests kek versions for services
     */
    'public setLatestKeks': function (kek_list) {
      this._kek_versions = kek_list;
      return this;
    },

    /**
     * Get the encrypted session key
     *
     * @return {string} encrypted session key
     */
    'public getSessionKeys': function () {
      return this._session_key;
    },

    /**
     * Set downstream system session keys
     */
    'public getDownstreamSessionKeys': function () {
      return this._downstream_session_keys;
    },

    /**
     * Get the latests kek versions for services
     */
    'public getLatestKeks': function () {
      return this._kek_versions;
    },
  });
