/**
 * Instantiate appropriate DataApi
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

const DataApiFactory = require('../../dapi/DataApiFactory');
const NodeHttpImpl = require('../../dapi/http/NodeHttpImpl');
const SpoofedNodeHttpImpl = require('../../dapi/http/SpoofedNodeHttpImpl');
const {Class} = require('easejs');
/**
 * Instantiates the appropriate DataApi object for the given service type
 */
module.exports = Class('DullahanClientDataApiFactory').extend(DataApiFactory, {
  /**
   * Origin URL
   *
   * @type {string}
   */
  'private _origin': '',

  /**
   * Spoofed user request
   *
   * @type {UserRequest}
   */
  'private _request': null,

  constructor(origin, request) {
    this._origin = '' + origin;
    this._request = request;
  },

  'override protected createHttpImpl'() {
    return NodeHttpImpl.use(SpoofedNodeHttpImpl(this._request))(
      {
        http: require('http'),
        https: require('https'),
      },
      require('url'),
      this._origin
    );
  },
});
