import {SecretStore} from './SecretStore';
import {LocalSecretStore} from './LocalSecretStore';
import {GcpSecretStore} from './GcpSecretStore';
import {SecretManagerServiceClient} from '@google-cloud/secret-manager';

export type SecretStoreOptions = 'gcp' | 'local';

export class SecretStoreFactory {
  constructor(
    private readonly gcp_file_path: string,
    private readonly project_id: string
  ) {}

  public generateSecretStore(store: SecretStoreOptions): SecretStore {
    if (store === 'local') {
      return new LocalSecretStore();
    }
    const gcp_client = new SecretManagerServiceClient({
      keyFile: this.gcp_file_path,
    });
    // Default to GCP
    return new GcpSecretStore(this.project_id, gcp_client);
  }
}
