/**
 * Commonly used typescript type guards
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
import {SecretStoreOptions} from '../../src/server/service/SecretStoreFactory';

export const isFulfilledPromise = <T extends CommonObject>(
  result: PromiseSettledResult<T>
): result is PromiseFulfilledResult<T> => {
  return result.status === 'fulfilled';
};

export const isRejectedPromise = <T extends CommonObject>(
  result: PromiseSettledResult<T>
): result is PromiseRejectedResult => {
  return !isFulfilledPromise(result);
};

export const isString = (x: unknown): string => {
  if (typeof x !== 'string') {
    throw new Error(`Expected string, received: ${x}`);
  }
  return x;
};

export const isArray = (x: unknown): unknown[] => {
  if (!Array.isArray(x)) {
    throw new Error(`Expected array, received: ${x}`);
  }
  return x;
};

export const isSecretStoreOption = (x: unknown): SecretStoreOptions => {
  if (x !== 'gcp' && x !== 'local') {
    throw new Error(`Expected SecretStoreOptions, received: ${x}`);
  }
  return x;
};
