import {PositiveInteger} from '../numeric';
import {Cmatch} from '../client/Cmatch';
import {Program} from '../program/Program';
import {DelayedStagingBucket} from '../bucket/DelayedStagingBucket';

type MessageGenerator = (
  messages: Record<string, unknown[]>,
  results: Record<string, unknown[]>
) => void;

export const validateProgramChange =
  (program: Program) =>
  (cmatch: Cmatch) =>
  (
    bucket: DelayedStagingBucket,
    diff: CommonObject,
    failures: CommonObject,
    getCurrentStepId: () => PositiveInteger,
    triggerValidation: () => void,
    generateMessages: MessageGenerator,
    validation_messages: Record<string, unknown[]>
  ) => {
    for (const name in diff) {
      // if we already have a problem with the field, then
      // save ourselves some effort and ignore it for now
      if (failures[name]) {
        continue;
      }

      const result = program.change(
        getCurrentStepId(),
        name,
        bucket,
        diff,
        cmatch.getMatches(),
        (...args) => {
          setTimeout(() => {
            // allow DOM operations to complete before we
            // trigger manipulations on it (TODO: this is
            // a workaround for group show/hide issues; we
            // need a better solution to guarantee order

            // eslint-disable-next-line
            triggerValidation.apply(null, <any>args);
          }, 25);
        }
      );

      for (const rname in result) {
        failures[rname] = [];

        for (const i in <any>result[rname]) {
          // the expected format is for it to contain the
          // value for each index
          (<any>failures[rname])[i] = (<any>result[rname])[i];
        }

        generateMessages(validation_messages, result);
      }
    }
  };

/**
 * Produce validation error messages intended for user display
 *
 * @param msg_dest - destination for messages per field and index
 * @param failures - failures per field name and index
 */
export const generateValidationMessages = (
  msg_dest: Record<string, unknown[]>,
  failures: Record<string, unknown[]>
) => {
  for (const rname in failures) {
    msg_dest[rname] = [];

    for (const i in failures[rname]) {
      msg_dest[rname][i] = failures[rname][i];
    }
  }
};
