/**
 * @license
 * Dollar formatter
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of liza.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Currency of the form `$%d`
 *
 * @see Currency
 *
 * @example
 *   Currency().parse( '$123' );      // => 123.00
 *   Currency().retrieve( '123.4' );  // => $123.40
 *
 * @class DollarFormatter
 * @mixes module:validate/formatter.Number
 * @mixes module:validate/formatter.StringFormat
 */
module.exports = require('./EchoFormatter')
  .use(require('./Number')(0))
  .use(require('./StringFormat')('$%s'));

// easejs/TS interop:
//   1. `extend({})` here finalizes the mixins above and produces a class
//      (constructor function) with all mixins applied; easejs normally does
//      this for you, but TS cannot.  From that point forward, you can use
//      inheritance from TS.
//   2. `exports.default` allows for `import DollarFormatter from ...`.
//      `require('./DollarFormatter`) will continue to return the easejs
//      class.
module.exports.default = module.exports.extend({});
