/**
 * Negatable dollar formatter
 *
 *  Copyright (C) 2010-2024 R-T Specialty, LLC.
 *
 *  This file is part of liza.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import DollarFormatter from './DollarFormatter';

/**
 * Formats and validates a positive or negative dollar value
 */
export class NegatableDollarFormatter extends DollarFormatter {
  parse(data: string): string {
    const parts = data.match(/^ *-(.*)$/);
    if (parts) {
      return '-' + super.parse(parts[1]);
    }

    return super.parse(data);
  }

  /**
   * Format a negatable number
   *
   * @param data - data to format
   *
   * @return formatted negatable number
   */
  retrieve(data: string): string {
    if (data.startsWith('-')) {
      return '-' + super.retrieve(data.substring(1));
    }

    return super.retrieve(data);
  }
}
