/**
 * Test bucket clearing
 *
 *  Copyright (C) 2010-2022 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import * as Sut from '../../src/bucket/BucketClear';
import {BucketData} from '../../src/bucket/Bucket';
import {expect} from 'chai';

describe('BucketMap', () => {
  describe('metaClearer', () => {
    interface BucketClearTest {
      label: string;
      input: BucketData;
      expected: BucketData;
      prefix?: string;
    }

    const tests: BucketClearTest[] = [
      {
        label: 'converts a clear update to target a bucket',
        input: {
          foo: ['bar', 'baz'],
          bar: ['bar'],
        },
        expected: {
          foo: ['', ''],
          bar: [''],
        },
      },
      {
        label: 'prefixes a value to the bucket keys',
        input: {
          foo: ['bar', 'baz'],
          bar: ['bar'],
        },
        expected: {
          'meta.foo': ['', ''],
          'meta.bar': [''],
        },
        prefix: 'meta.',
      },
      {
        label: 'converts a multi-level clear update to target a bucket',
        input: {
          foo: [['bar'], 'baz'],
          bar: ['bar'],
        },
        expected: {
          foo: ['', ''],
          bar: [''],
        },
      },
      {
        label: 'does not empty-clear fields',
        input: {
          foo: ['bar', 'baz'],
          bar: ['bar'],
          baz: [],
        },
        expected: {
          foo: ['', ''],
          bar: [''],
        },
      },
      {
        label: 'converts an empty clear update to target a bucket',
        input: {},
        expected: {},
      },
    ];

    tests.forEach((test: BucketClearTest) => {
      const {label, input, expected, prefix} = test;

      it(label, () => {
        expect(Sut.bucketClearer(prefix)(input)).to.deep.equal(expected);
      });
    });
  });
});
