import {ClientDataProxy} from '../../../src/client/ClientDataProxy';
import {ClientQuote} from '../../../src/client/quote/ClientQuote';
import {Client} from '../../../src/client/Client';
import {FetchRateEventHandler as Sut} from '../../../src/client/event/FetchRateEventHandler';
import {expect} from 'chai';

describe('FetchRateEventHandler', () => {
  describe('#handle', () => {
    it('fetches rate results', () => {
      const sut = getSut();
      let data = {};

      const expected = {
        foo: 'bar',
      };

      const callback = (_: any, d: any) => (data = d);

      sut.handle('fetchRate', callback, {action: 'fetchRate'});

      expect(data).to.deep.equal(expected);
    });
  });
});

const getSut = () => {
  return new Sut(getClient(), getProxy());
};

const getClient = () => {
  return <Client>(<unknown>{
    getQuote,

    isSaving: () => false,

    getUi: () => ({getStep}),
  });
};

const getProxy = () => {
  return <ClientDataProxy>(<unknown>{
    get(_uri: any, callback: any) {
      const response = {content: {data: {foo: 'bar'}}};
      callback(response);
    },
  });
};

const getQuote = () => <ClientQuote>(<unknown>{
    getId: () => 1,
    getCurrentStepId: () => 1,
    refreshData: () => {},
    setData: () => {},
    setInitialRatedDate: () => {},
    setLastPremiumDate: () => {},
  });

const getStep = () => ({
  invalidate: () => {},
  emptyBucket: () => {},
});
