/**
 *  Tests for Step Simulator
 *
 *  Copyright (C) 2010-2022 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
import {Cmatch} from '../../../src/client/Cmatch';
import {PositiveInteger} from '../../../src/numeric';
import {Program} from '../../../src/program/Program';
import {ServerSideQuote} from '../../../src/server/quote/ServerSideQuote';
import {expect} from 'chai';
import {
  StepSimulator as Sut,
  StepSummary,
} from '../../../src/dullahan/step/StepSimulator';

describe('StepSimulator', () => {
  describe('#each', () => {
    it('walks each step', () => {
      let count = 0;

      const sut = getSut();

      return sut
        .each(() => count++, getCmatch())
        .then(() => {
          // note: this does not include step 4 because we do not want to
          // do work on the "publish_quote_ind" field
          expect(count).to.eq(3);
        });
    });

    it('gets all steps', () => {
      // note: this does not include "publish_quote_ind" by design
      const expected: StepSummary[] = [
        {
          fields: ['foo'],
          id: <PositiveInteger>1,
        },
        {
          fields: [],
          id: <PositiveInteger>2,
        },
        {
          fields: ['bar'],
          id: <PositiveInteger>3,
        },
      ];

      const sut = getSut();

      return sut.getSteps(getCmatch()).then(steps => {
        expect(steps).to.deep.equal(expected);
      });
    });
  });
});

const getCmatch = () => <Cmatch>(<unknown>{
    getMatches: () => ({}),
  });

const getProgram = () => <Program>(<unknown>{
    getFirstStepId: () => 1,
    isStepVisible: () => true,
    qstep: {foo: 1, bar: 3, publish_quote_ind: 4},
  });

const getQuote = () => <ServerSideQuote>(<unknown>{
    classify: () => {},
    getFieldState: () => ({foo: {}, bar: {}, publish_quote_ind: {}}),
  });

const getSut = () =>
  new Sut(getQuote(), getProgram(), ['foo', 'bar', 'publish_quote_ind']);
