import {init as sut} from '../../src/system/Tracing';
import {NodeTracerProvider} from '@opentelemetry/node';
import {expect} from 'chai';

describe('init', () => {
  let span_added = false;
  let registered = false;

  beforeEach(() => {
    span_added = false;
    registered = false;
  });

  const provider = <NodeTracerProvider>(<unknown>{
    addSpanProcessor: () => {
      span_added = true;
    },
    register: () => {
      registered = true;
    },
    getTracer: () => {},
  });

  it('empty env', () => {
    const env = {};

    sut(provider, env);
    expect(span_added).to.be.false;
    expect(registered).to.be.false;
  });

  it('explicitly disabled', () => {
    const env = {
      OPENTELEMETRY_COLLECTOR_EXPORTER_ENABLED: 0,
    };

    sut(provider, <NodeJS.ProcessEnv>(<unknown>env));
    expect(span_added).to.be.false;
    expect(registered).to.be.false;
  });

  it('enabled', () => {
    const env = {
      OPENTELEMETRY_COLLECTOR_EXPORTER_ENABLED: 1,
    };

    sut(provider, <NodeJS.ProcessEnv>(<unknown>env));
    expect(span_added).to.be.true;
    expect(registered).to.be.true;
  });
});
