/**
 * Test case for FieldsStoreTest
 *
 *  Copyright (C) 2010-2022 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework
 *
 *  Liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import {QuestionTypes, SelectData} from '../../../src/program/Program';
import {FieldsStore as Sut} from '../../../src/ui/context/FieldsStore';
import {PositiveInteger} from '../../../src/numeric';

import {expect} from 'chai';

describe('FieldsStore', () => {
  (<
    {
      label: string;
      qtypes: QuestionTypes;
      qdata: SelectData;
      name: string;
      value: string;
      expected: string;
    }[]
  >[
    {
      label: 'gets select label for number value',
      qtypes: {baz: {type: 'text', dim: <PositiveInteger>0}},
      qdata: {baz: {0: 'zoo', 1: 'foo', 2: 'bar'}},
      name: 'baz',
      value: '0',
      expected: 'zoo',
    },
    {
      label: 'gets select label for string value',
      qtypes: {bar: {type: 'text', dim: <PositiveInteger>0}},
      qdata: {bar: {foo: 'baz', zoo: 'foo', boo: 'bar'}},
      name: 'bar',
      value: 'boo',
      expected: 'bar',
    },
  ]).forEach(({label, qtypes, qdata, name, value, expected}) => {
    it(label, () => {
      const sut = new Sut(qtypes, qdata);
      const given = sut.getSelectLabel(name, value);
      expect(given).to.equal(expected);
    });
  });

  it('setSelectLabel adds a label for a new value', () => {
    const qtypes = {baz: {type: 'text', dim: <PositiveInteger>0}};
    const qdata = {foo: {}};
    const name = 'foo';
    const value = 'bar';
    const label = 'baz';
    const sut = new Sut(qtypes, qdata);

    sut.setSelectLabel(name, value, label);
    const given = sut.getSelectLabel(name, value);
    expect(given).to.equal(label);
  });

  it('getQuestionType gets type for a field', () => {
    const name = 'baz';
    const qtypes = {baz: {type: 'text', dim: <PositiveInteger>0}};
    const qdata = {foo: {}};
    const sut = new Sut(qtypes, qdata);

    const given = sut.getQuestionType(name);
    expect(given).to.equal('text');
  });
});
