/**
 * Test case for LiveSearchFieldStyler
 *
 *  Copyright (C) 2010-2022 R-T Specialty, LLC.
 *
 *  This file is part of the Liza Data Collection Framework
 *
 *  Liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import {LiveSearchFieldStyler as Sut} from '../../../../src/ui/context/styler/LiveSearchFieldStyler';
import {ContextContent} from '../../../../src/ui/context/FieldContext';
import {PositiveInteger} from '../../../../src/numeric';
import {Renderer} from 'react-dom';
import {
  LiveSearch,
  LiveSearchProps,
} from '../../../../src/ui/question/LiveSearch';

import {expect} from 'chai';

before(function () {
  this.jsdom = require('jsdom-global')();
});

after(function () {
  this.jsdom();
});

describe('LiveSearchFieldStyler', () => {
  it('Renders a react component', () => {
    const anchor = document.createElement('div');
    const input = document.createElement('input');
    const select = document.createElement('select');
    const createElement = () => {};
    const render = () => {};

    const content = <ContextContent>(<unknown>{
      querySelector: () => anchor,
      parentNode: {
        querySelector: (query: string) => {
          if (query === "[id='q_foo_1']") {
            return select;
          } else {
            return input;
          }
        },
      },
    });

    const sut = new Sut(
      'foo',
      <PositiveInteger>1,
      <Document>(<unknown>{
        createEvent: () => {
          return {
            initEvent: () => {},
          };
        },
      }),
      <CreateElement<LiveSearchProps, LiveSearch>>(<unknown>createElement),
      <Renderer>(<unknown>render)
    );

    sut.setValue(content, '');

    expect(anchor.innerHTML).to.not.be.undefined;
  });

  it('Disables a react component', () => {
    const disabled = true;
    const anchor = document.createElement('div');
    const input = document.createElement('input');
    const select = document.createElement('select');
    const createElement = () => {};
    const render = () => {};

    const content = <ContextContent>(<unknown>{
      querySelector: () => anchor,
      parentNode: {
        querySelector: (query: string) => {
          if (query === "[id='q_foo_1']") {
            return select;
          } else {
            return input;
          }
        },
      },
    });

    const sut = new Sut(
      'foo',
      <PositiveInteger>1,
      <Document>(<unknown>{
        createEvent: () => {
          return {
            initEvent: () => {},
          };
        },
      }),
      <CreateElement<LiveSearchProps, LiveSearch>>(<unknown>createElement),
      <Renderer>(<unknown>render)
    );

    sut.disable(content, disabled);

    expect(anchor.innerHTML).to.not.be.undefined;
  });
});
