/**
 * Test classification utility functions
 *
 *  Copyright (C) 2010-2022 R-T Specialty, LLC.
 *
 *  This file is part of liza.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import {expect} from 'chai';

import {ClassificationResult} from '../../src/program/Program';

import {
  forEachAggregatedClass as sutForEach,
  forEachAggregatedClassSuffix as sutForEachPrefix,
} from '../../src/util/classifications';

describe('#forEachAggregatedClass', () => {
  type SutForEachTestCase = {
    label: string;
    classes: ClassificationResult;
    name: string;
    expected: string[];
  };

  (<SutForEachTestCase[]>[
    {
      label: 'Nothing is returned if aggregate does not exist',
      classes: {},
      name: 'foo',
      expected: [],
    },
    {
      label: 'Nothing is returned if aggregate is false',
      classes: {
        foo: {is: false},
      },
      name: 'foo',
      expected: [],
    },
    {
      label: 'Nothing is returned if classes are false',
      classes: {
        foo: {is: true},
      },
      name: 'foo',
      expected: [],
    },
    {
      label: 'Returns only true classifications',
      classes: {
        foo: {is: true},
        'foo-bar': {is: true},
        'foo-baz': {is: false},
      },
      name: 'foo',
      expected: ['foo-bar'],
    },
  ]).forEach(({label, classes, name, expected}) => {
    it(label, () => {
      const given: string[] = [];

      sutForEach(classes, name, key => given.push(key));

      expect(given).to.deep.equal(expected);
    });
  });
});

describe('#forEachAggregatedClass', () => {
  it('returns only the suffix of classes', () => {
    const given: string[] = [];

    sutForEachPrefix(
      {
        foo: {is: true},
        'foo-bar': {is: true},
        'foo-baz': {is: false},
      },
      'foo',
      key => given.push(key)
    );

    expect(given).to.deep.equal(['bar']);
  });
});
