/**
 * @license
 * domain formatter test
 *
 *  Copyright (C) 2010-2022 R-T Specialty, LLC.
 *
 *  This file is part of liza.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

var liza = require('../../../'),
  Sut = liza.validate.formatter.DomainFormatter,
  assert = require('assert');

describe('validate.formatter.DomainFormatter', function () {
  [
    {domain: 'domain.com', expected: 'domain.com'},
    {domain: 'this-domain.com', expected: 'this-domain.com'},
    {domain: 'https://www.domain.com', expected: 'domain.com'},
    {domain: 'http://www.domain.com', expected: 'domain.com'},
    {domain: 'www.domain.com', expected: 'domain.com'},
    {domain: 'https://www.domain.com:80', expected: 'domain.com'},
    {domain: 'https://www.domain.com/foo', expected: 'domain.com'},
    {domain: 'https://api.domain.com', expected: 'api.domain.com'},
    {domain: 'http://api.domain.com', expected: 'api.domain.com'},
    {domain: 'api.domain.com', expected: 'api.domain.com'},
    {domain: 'http://api.domain.com:80', expected: 'api.domain.com'},
    {domain: 'http://api.domain.com:80/foo', expected: 'api.domain.com'},
    {domain: 'DOMAIN.COM', expected: 'DOMAIN.COM'},
    {domain: 'DOMAIN.COM/FOO', expected: 'DOMAIN.COM'},
    {
      domain: 'api.domain.travelersinsurance',
      expected: 'api.domain.travelersinsurance',
    },
  ].forEach(({domain, expected}) => {
    it(`should format valid data '${domain}' to '${expected}'`, function () {
      assert.equal(Sut.parse(domain), expected);
    });
  });

  [
    '',
    'domain',
    'domain.domaindomaindomaindomaindomain',
    'domain:domain',
  ].forEach(domain => {
    it(`should throw errors for invalid data '${domain}'`, function () {
      assert.throws(() => Sut.parse(domain), Error);
    });
  });
});
