/**
 * @license
 * Negatable dollar formatter test
 *
 *  Copyright (C) 2010-2022 R-T Specialty, LLC.
 *
 *  This file is part of liza.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import {assert, expect} from 'chai';
import {NegatableDollarFormatter as Sut} from '../../../src/validate/formatter/NegatableDollarFormatter';

describe('validate.formatter.NegatableDollarFormatter', () => {
  describe('parse', () => {
    Object.entries({
      'no change single digit': ['1', '1'],
      'no change two digits': ['12', '12'],
      'no change three digits': ['123', '123'],
      'no change four digits': ['1234', '1234'],
      'no change five digits': ['12345', '12345'],
      'no change single digit, negative': ['-1', '-1'],
      'no change two digits, negative': ['-12', '-12'],
      'no change three digits, negative': ['-123', '-123'],
      'no change four digits, negative': ['-1234', '-1234'],
      'no change five digits, negative': ['-12345', '-12345'],
      'truncate positive number': ['  1337  ', '1337'],
      'truncate negative number': ['  -1337  ', '-1337'],
      'truncate single decimal': ['444.4', '444'],
      'truncate two decimals': ['444.44', '444'],
      'truncate decimals, no rounding': ['444.66', '444'],
    }).forEach(([label, [given, expected]]) =>
      it(label, () => assert.equal(new Sut().parse(given), expected))
    );

    Object.entries({
      'not a number': ['foo', 'Invalid number'],
      'number with characters': ['123foo', 'Invalid number'],
    }).forEach(([label, [given, expected]]) =>
      it(label, () => {
        expect(() => new Sut().parse(given)).to.throw(expected);
      })
    );
  });

  describe('retrieve', () => {
    it('returns formatted negative dollars without changing its value', () => {
      assert.equal(new Sut().retrieve('-9'), '-$9');
      assert.equal(new Sut().retrieve('-98'), '-$98');
      assert.equal(new Sut().retrieve('-987'), '-$987');
      assert.equal(new Sut().retrieve('-9876'), '-$9,876');
      assert.equal(new Sut().retrieve('-98765'), '-$98,765');
      assert.equal(new Sut().retrieve('-987654'), '-$987,654');
      assert.equal(new Sut().retrieve('-9876543'), '-$9,876,543');
      assert.equal(new Sut().retrieve('-98765432'), '-$98,765,432');
      assert.equal(new Sut().retrieve('-987654321'), '-$987,654,321');
    });

    it('returns formatted positive dollars without changing its value', () => {
      assert.equal(new Sut().retrieve('9'), '$9');
      assert.equal(new Sut().retrieve('98'), '$98');
      assert.equal(new Sut().retrieve('987'), '$987');
      assert.equal(new Sut().retrieve('9876'), '$9,876');
      assert.equal(new Sut().retrieve('98765'), '$98,765');
      assert.equal(new Sut().retrieve('987654'), '$987,654');
      assert.equal(new Sut().retrieve('9876543'), '$9,876,543');
      assert.equal(new Sut().retrieve('98765432'), '$98,765,432');
      assert.equal(new Sut().retrieve('987654321'), '$987,654,321');
    });
  });
});
