/**
 * @license
 * StringFormat formatter test
 *
 *  Copyright (C) 2010-2022 R-T Specialty, LLC.
 *
 *  This file is part of liza.
 *
 *  liza is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

const liza = require('../../../'),
  Sut = liza.validate.formatter.PhoneFormatter,
  assert = require('assert');

describe('validate.formatter.PhoneFormatter', () => {
  describe('parse', () => {
    [
      ['212-555-1212', '2125551212'],
      ['212 555 1212', '2125551212'],
      ['212.555.1212', '2125551212'],
      ['2125551212', '2125551212'],
      ['1212-555-1212', '2125551212'],
      ['1212.555.1212', '2125551212'],
      ['12125551212', '2125551212'],
      ['1-212-555-1212', '2125551212'],
      ['1 212 555 1212', '2125551212'],
      ['1.212.555.1212', '2125551212'],
      ['+1212-555-1212', '2125551212'],
      ['+1212.555.1212', '2125551212'],
      ['+12125551212', '2125551212'],
      ['+1-212-555-1212', '2125551212'],
      ['+1 212 555 1212', '2125551212'],
      ['+1.212.555.1212', '2125551212'],
    ].forEach(test_data => assert.equal(Sut.parse(test_data[0]), test_data[1]));

    ['', 'plainstring', '5551212', '212/555/1212'].forEach(phone_number =>
      assert.throws(() => Sut.parse(phone_number), Error)
    );
  });

  describe('retrieve', () => {
    [
      ['212-555-1212', '212-555-1212'],
      ['212 555 1212', '212 555 1212'],
      ['212.555.1212', '212.555.1212'],
      ['2125551212', '212-555-1212'],
      ['1212-555-1212', '1212-555-1212'],
      ['', ''],
    ].forEach(test_data =>
      assert.equal(Sut.retrieve(test_data[0]), test_data[1])
    );
  });
});
